/*
 *      Program do przegladania logow z ekg.
 *      Wczytywanie logow.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __DBASE_HH__
#define __DBASE_HH__

#include <vector>
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>

#include "files.hh"
#include "config.hh"


struct message {
    message() {}
    message(const int &_type, const int &_time, const int &_rtime, const char *_str):
	type(_type), time(_time), rtime(_rtime), str(_str), highlighted(false) {}

    int type;
    int time;
    int rtime;
    std::string str;
    bool highlighted;
};
//enum { MSG_RECV, MSG_SEND } msg_type;

#define __diff(x,y) ((x)<(y) ? ((y)-(x)) : ((x)-(y)))

class chat_t {
public:
    chat_t():_msgs(0) {}

    bool belongs(const int time) const {
	if(!_msgs.size())
	    return true;
	
	return (__diff(_msgs.back().time, time) < cfg.time_diff());
    }

    void add_msg(const int type, const int time,
		 const int rtime, const char *str);

    int size(void) const { return _msgs.size(); }

    int start(void) const { return _msgs[0].time; }
    int end(void) const { return _msgs.back().time; }

    const message & operator[](int i) const { return _msgs[i]; }
    message & get_message(int i) { return _msgs[i]; }

protected:
    std::vector<message> _msgs;
};



class Chat : public chat_t {
public:
    Chat(const std::string &nick, int uin, chat_t &chat):chat_t(chat),_uin(uin),_nick(nick) {}

    const std::string & nick(void) const { return _nick; }
    int uin(void) const { return _uin; }
    
protected:
    int _uin;
    std::string _nick;
};



struct uinlog_t {
    uinlog_t(const int &uin, const char *nick):_uin(uin),_msg_no(0),_nick(nick),_change_nick(true) {}
    int size(void) { return chats.size(); }

    int _uin, _msg_no;
    std::string _nick;
    bool _change_nick;
    std::vector<chat_t> chats;

    Chat get_Chat(unsigned int i) { return Chat(nick(),uin(),chats[i]); }
    chat_t &get_chat(unsigned int i) { return chats[i]; }
    const chat_t & operator[](unsigned int i) { return chats[i]; }
    int uin(void) { return _uin; }
    std::string nick(void) { return _nick; }
    bool change_nick(void) { return _change_nick; }
};

bool operator<(const uinlog_t &a, const uinlog_t &b);





class dbase_t {
public:
    void read_file(const std::string &name);
    void add_msg(const int type, const int uin, const char *nick,
		 const int time, const int rtime, const char *str);

    int size(void) { return uinlogs.size(); }

    uinlog_t *get_uinlog(unsigned int i) { return &uinlogs[i]; }

    void set_uins(const std::map<int, std::string>& arg) { uins = arg; }

    void sort();

private:
    std::vector<uinlog_t> uinlogs;
    std::map<int, std::string> uins;
};

#endif /* __DBASE_HH__ */
