/*
 *      Program do przegladania logow z ekg.
 *      Klasa obslugi wejscia z klawiatury.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __INPUT_HH__
#define __INPUT_HH__

extern "C" {
    #include <ncurses.h>
}

#include "errors.hh"
#include "stuff.hh"
#include "command.hh"
#include "com_various.hh"
#include "window.hh"


/* klasa wyjatku rzucana w przypadku wpisania niepoprawnej komendy */
struct bad_command : public ekglog_exception { 
    bad_command(const std::string &str):_str(str) {}

    const std::string &str(void) { return _str; }

    std::string _str;
};

class input : public window_base {
public:
    input(int startx, int starty, int width);
    ~input(void);

    void add_key(int key);
    void refresh(void);

    int getkb(void)
	{ return wgetch(_win); }

    void active(void) {}
    void passive(void) {}

    void up(void);
    void down(void);
    void pgup(void);
    void pgdown(void);

    void register_command(command_base *c);

    std::string _line;  /* tresc wpisywanej komendy */

private:

    void parse_command(void);
    void execute_command(void);

    void add_to_history(std::string str) {
	_history.push_back(str);
	if(_history.size() == 100)
	    _history.erase(_history.begin());
	_act_line = _history.size();
    }

    WINDOW *_win;       /* okno ncurses */
    int _width;         /* szerokosc okna */

    std::vector<std::string> _argv;
			/* wektor argumentow dla komendy */
    std::vector<command_base*> _commands;
			/* wektor zarejestrowanych komend */
    std::vector<std::string> _history;
    unsigned int _act_line;
};


#endif /* __INPUT_HH__ */
