/*
 *      Program do przegladania logow z ekg.
 *      Przydatne rzeczy.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "stuff.hh"

int str2int(const std::string &str) {
    std::istringstream tmpstr(str);
    int tmpint;
    tmpstr >> tmpint;
    if(!tmpstr)
	throw bad_string();

    return tmpint;
}


void print_line(WINDOW *w, const std::string &str) {
    wprintw(w, str.c_str());
}


/*
 * color_pair()
 *
 * zwraca pare kolorow odpowiadajaca atrybutom
 * wzorowane na ekg
 *
 *  - fg - kolor czcionki
 *  - bold - ew. pogrubienie czcionki
 *  - bg - kolor tla
 */
int color_pair(int fg, int bold, int bg) {
    if( fg >= 8 ) {
	bold = 1;
	fg &= 7;
    }
    if( fg == COLOR_BLACK && bg == COLOR_BLACK ) {
	fg = 7;
    }
    else if( fg == COLOR_WHITE && bg == COLOR_BLACK ) {
	fg = 0;
    }

    return COLOR_PAIR(fg + 8 * bg) | ((bold) ? A_BOLD : 0);
}


/*
 * line_lenght()
 *
 * zwraca rzeczywista ilosc znakow w stringu - czyli pomija
 * wszelkie znaki formatujace oraz niewyswietlane \
 *
 *  - str - string do zanalizowania
 */
int line_lenght(const std::string &str) {
    unsigned int i=0;
    int len=0;
    while(i < str.size()) {
	switch(str[i]) {
	case '\\':
	    switch(str[++i]) {
		case 'c':
		    i+=4;
		    break;
		case 'd':
		    ++i;
		default:
		    ++i;
		    ++len;
		    break;
	    }
	    break;
	default:
	    ++i;
	    ++len;
	    break;
	}
    }

    return len;
}


/*
 * time_string()
 *
 * zwraca string ze sformatowanym czasem
 *
 *  - t - czas to sformatowania
 */
const std::string time_string(time_t t) {
    struct tm *st;
    std::string tmp;

    st = localtime(&t);
    
    tmp += add_zero(st->tm_hour);
    tmp += ':';
    tmp += add_zero(st->tm_min);

    return tmp;
}


/*
 * add_zero()
 *
 * dodaje zero do int'a, jesli ma tylko jedna cyfre
 *
 *  - arg - liczba do sformatowania
 */
const char *add_zero(int arg) {
    static char buf[3];
    if(arg<10) {
	buf[0] = '0';
	buf[1] = arg + '0';
    }
    else {
	buf[0] = arg/10 + '0';
	buf[1] = arg%10 + '0';
    }
    buf[2] = 0;
    return buf;
}


/*
 * format_date()
 *
 * formatuje date i czas do stringa i zwraca wskaznik na niego
 *
 *  - t - czas do sformatowania
 */
const char *format_date(time_t t) {
    static char buf[30];
    struct tm *st = localtime(&t);
    char min[3],hour[3];

    strcpy(min,add_zero(st->tm_min));
    strcpy(hour,add_zero(st->tm_hour));

    sprintf(buf,"%d.%d.%d, %s:%s", 1900+st->tm_year, 1+st->tm_mon,
	    st->tm_mday, hour, min);

    return buf;
}


/*
 * wprintl()
 *
 * wypisuje na ekran linie sformatowana wg. zasad z ekglog
 *
 *  - w - okno ncurses, to ktorego piszemy
 *  - str - linia do wypisania
 *  - defcolor - domyslne kolory okna
 */
void wprintl(WINDOW *w, const std::string &str, int defcolor) {
    unsigned int k=0;
    int color = defcolor==0?color_pair(7,0,0):defcolor;
    wattron(w,color);

    while(k < str.size()) {
	switch(str[k]) {
	    case '\\':
		if(++k == str.size()) {
		    wprintw(w, "\\");
		}
		else {
		    switch(str[k]) {
			case 'c':
			    wattroff(w, color);
			    color = color_pair((str[k+1]-'0'), (str[k+2]-'0'), (str[k+3]-'0'));
			    wattron(w, color);
			    k+=4;
			    break;
			case 'd':
			    wattroff(w, color);
			    color = defcolor;
			    wattron(w, color);
			    ++k;
			    break;
			case 'n':
			    waddch(w, '\n');
			    ++k;
			    break;
			case 'r':
			    ++k;
			    break;
			case '\'':
			case '\\':
			case '\"':
			    wprintw(w, "%c", str[k]);
			    ++k;
			    break;
			default:
			    wprintw(w, "\\%c", str[k]);
			    ++k;
			    break;
		    }
		    break;
		default:
		    wprintw(w, "%c", str[k]);
		    ++k;
		    break;
	    }/* switch(*k) */
	}/* else */
    }/* while() */
    wattroff(w, color);
    wattron(w, defcolor);
}
